package ru.dbotthepony.mc.otm.data

import com.mojang.datafixers.util.Pair
import com.mojang.serialization.Codec
import com.mojang.serialization.DataResult
import com.mojang.serialization.DynamicOps
import java.util.UUID
import java.util.stream.LongStream

object UUIDCodec : Codec<UUID> {
	override fun <T : Any> encode(input: UUID, ops: DynamicOps<T>, prefix: T): DataResult<T> {
		return DataResult.success(ops.createLongList(LongStream.of(input.mostSignificantBits, input.leastSignificantBits)))
	}

	override fun <T : Any> decode(ops: DynamicOps<T>, input: T): DataResult<Pair<UUID, T>> {
		return ops.getLongStream(input).flatMap {
			val l = it.limit(2).toArray()

			if (l.size != 2) {
				DataResult.error { "Can't construct UUID from ${l.size} elements" }
			} else {
				DataResult.success(Pair(UUID(l[0], l[1]), ops.empty()))
			}
		}
	}
}
